@@ -1,10 +1,13 @@
 /*!
- * Copyright 2019 XGBoost contributors
+ * Copyright 2019-2021 XGBoost contributors
  */
 #ifndef XGBOOST_DATA_SIMPLE_BATCH_ITERATOR_H_
 #define XGBOOST_DATA_SIMPLE_BATCH_ITERATOR_H_
 
-#include <xgboost/data.h>
+#include <memory>
+#include <utility>
+
+#include "xgboost/data.h"
 
 namespace xgboost {
 namespace data {
@@ -12,20 +15,21 @@ namespace data {
 template<typename T>
 class SimpleBatchIteratorImpl : public BatchIteratorImpl<T> {
  public:
-  explicit SimpleBatchIteratorImpl(T* page) : page_(page) {}
-  T& operator*() override {
-    CHECK(page_ != nullptr);
-    return *page_;
-  }
+  explicit SimpleBatchIteratorImpl(std::shared_ptr<T const> page) : page_(std::move(page)) {}
   const T& operator*() const override {
     CHECK(page_ != nullptr);
     return *page_;
   }
-  void operator++() override { page_ = nullptr; }
+  SimpleBatchIteratorImpl &operator++() override {
+    page_ = nullptr;
+    return *this;
+  }
   bool AtEnd() const override { return page_ == nullptr; }
 
+  std::shared_ptr<T const> Page() const override { return page_; }
+
  private:
-  T* page_{nullptr};
+  std::shared_ptr<T const> page_{nullptr};
 };
 
 }  // namespace data
